@@ -1,6 +1,6 @@
 <?php
 /**
- * Copyright 2002-2016 Horde LLC (http://www.horde.org/)
+ * Copyright 2002-2017 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.horde.org/licenses/lgpl21.
@@ -16,7 +16,7 @@
  *
  * @author    Chuck Hagenbuch <chuck@horde.org>
  * @category  Horde
- * @copyright 2002-2016 Horde LLC
+ * @copyright 2002-2017 Horde LLC
  * @license   http://www.horde.org/licenses/lgpl21 LGPL-2.1
  * @package   Image
  */
@@ -32,8 +32,8 @@ class Horde_Image_Effect_Im_Border extends Horde_Image_Effect_Border
     public function apply()
     {
         $this->_image->addPostSrcOperation(sprintf(
-            "-bordercolor \"%s\" %s -border %s",
-            $this->_params['bordercolor'],
+            '-bordercolor "%s" %s -border %d',
+            escapeshellarg($this->_params['bordercolor']),
             (!empty($this->_params['preserve']) ? '-compose Copy' : ''),
             $this->_params['borderwidth']
         ));
