@@ -139,7 +139,7 @@ class Horde_Image_Im extends Horde_Image_Base
                 $this->_width,
                 $this->_height,
                 escapeshellarg($this->_background),
-                $this->_type
+                escapeshellarg($this->_type)
             );
             $this->executeConvertCmd($cmd);
         }
@@ -200,7 +200,7 @@ class Horde_Image_Im extends Horde_Image_Base
         $command = $this->_convert . ' ' . implode(' ', $this->_operations)
             . ' "' . $tmpin . '"\'[' . (integer)$options['index'] . ']\' '
             . implode(' ', $this->_postSrcOperations)
-            . ' -strip ' . $this->_type . ':"' . $tmpout . '" 2>&1';
+            . ' -strip ' . escapeshellarg($this->_type) . ':"' . $tmpout . '" 2>&1';
         $this->_logDebug(sprintf("convert command executed by Horde_Image_im::raw(): %s", $command));
         exec($command, $output, $retval);
         if ($retval) {
@@ -368,20 +368,46 @@ class Horde_Image_Im extends Horde_Image_Base
      */
     public function text(
         $string, $x, $y, $font = '', $color = 'black', $direction = 0,
-        $fontsize = 'small'
-    )
+        $fontsize = 'small')
     {
-        $string = addslashes('"' . $string . '"');
+        $string = $this->_escapeMvgString($string);
+
         $fontsize = Horde_Image::getFontSize($fontsize);
         $command = 'text ' . (integer)$x . ',' . (integer)$y . ' ' . $string;
         $this->_postSrcOperations[] = '-fill ' . escapeshellarg($color)
             . (!empty($font) ? ' -font ' . escapeshellarg($font) : '')
             . sprintf(
-                ' -pointsize %d -gravity northwest -draw "%s" -fill none',
+                ' -pointsize %d -gravity northwest -draw \'%s\' -fill none',
                 $fontsize, $command
             );
     }
 
+    /**
+     * Escape a string used as the MVG string parameter to the Im -draw
+     * command when the -draw command's argument is surronded by single quotes.
+     *
+     * http://www.imagemagick.org/Usage/draw/#quote
+     *
+     * @param string $string  The string to escape.
+     *
+     * @return  The string escaped for use with a draw command.
+     */
+    protected function _escapeMvgString($string)
+    {
+        // First escape any existing backslashes - before we add our own.
+        $string = str_replace('\\', '\\\\', $string);
+
+        // Now see if we need to deal with single quotes, which need to
+        // be escaped, but outside the shell's single quote mode.
+        $string = str_replace("'", "'\''", $string);
+
+        // The only other characters that need to be escaped are double quotes
+        //
+        $string = str_replace('"', '\\"', $string);
+
+        return sprintf('"%s"', $string);
+    }
+
     /**
      * Draws a circle.
      *
@@ -772,21 +798,41 @@ class Horde_Image_Im extends Horde_Image_Base
      * Request a specific image from the collection of images.
      *
      * @param integer $index  The index to return
-     *
+     * @param boolean $flatten  If true, flatten the returned image using
+     *                          $bgColor as the background color. Useful
+     *                          for PDF files to remove transparency before
+     *                          rendering.
+     * @param string $bgColor   The background color to use when flattening the
+     *                          image.
      * @return Horde_Image_Base
      */
-    public function getImageAtIndex($index)
+    public function getImageAtIndex($index, $flatten = false, $bgColor = 'white')
     {
         $this->_logDebug('Horde_Image_Im#getImageAtIndex: ' . $index);
-        if ($index >= $this->getImagePageCount()) {
+
+        if ($index > 0 && $index >= $this->getImagePageCount()) {
             throw new Horde_Image_Exception('Image index out of bounds.');
         }
         $rawImage = $this->_raw(true, array('index' => $index, 'preserve_data' => true));
         $image = new Horde_Image_Im(array('data' => $rawImage), $this->_context);
 
+        if ($flatten) {
+            $image->flattenImage($bgColor);
+        }
         return $image;
     }
 
+    /**
+     * Flatten this image and remove transparency.
+     *
+     * @param string $bgColor  The background color to use.
+     */
+    public function flattenImage($bgColor = 'white')
+    {
+        $this->_postSrcOperations[] = sprintf('-background %s -flatten', $bgColor);
+        $this->raw(false, array('stream' => true));
+    }
+
     /**
      * Return the number of image pages available in the image object.
      *
@@ -795,8 +841,7 @@ class Horde_Image_Im extends Horde_Image_Base
     public function getImagePageCount()
     {
         if (is_null($this->_pages)) {
-            $pages = $this->_getImagePages();
-            $this->_pages = array_pop($pages);
+            $this->_pages = $this->_getImagePages();
         }
         $this->_logDebug('Horde_Image_Im#getImagePageCount: ' . $this->_pages);
 
@@ -808,11 +853,15 @@ class Horde_Image_Im extends Horde_Image_Base
     {
         $this->_logDebug('Horde_Image_Im#_getImagePages');
         $filename = $this->toFile();
-        $cmd = $this->_identify . ' -format "%n" ' . $filename;
+        $cmd = $this->_identify . ' -format "%n " ' . $filename;
+        $this->_logDebug($cmd);
         exec($cmd, $output, $retval);
         if ($retval) {
             $this->_logErr(sprintf("Error running command: %s", $cmd . "\n" . implode("\n", $output)));
         }
+        $output = array_pop($output);
+        $output = substr($output, 0, strpos($output, ' ') + 1);
+        $this->_logDebug(print_r($output, true));
         unlink($filename);
 
         return $output;
